#ifdef WIN32
#define _CRT_SECURE_NO_WARNINGS
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "util.h"

char	g_filename[2048];

void
error_exit(int code)
{
	fprintf(stderr,"Exit code %d\n", code);
	exit(code);
}

void
outofmemory(void)
{
	fprintf(stderr, "Out of memory\n");
	error_exit(4);
}

void *
xmalloc(int size)
{
	void *P;

	P = malloc(size);
	if (!P) {
		outofmemory();
	}
	return(P);
}

void *
xrealloc(void *oldP, int size)
{
	void *P;

	P = realloc(oldP, size);
	if (!P) {
		outofmemory();
	}
	return(P);
}

char	label_char[256]    = {0};
char	function_char[256] = {0};

void
init_lookup_chars(void)
{
	int		i;

	for (i = '0'; i <= '9'; ++i) {
		label_char[i]    = 1;
		function_char[i] = 1;
	}
	for (i = 'A'; i <= 'Z'; ++i) {
		label_char[i]    = 2;
		function_char[i] = 2;
	}
	for (i = 'a'; i <= 'z'; ++i) {
		label_char[i]    = 3;
		function_char[i] = 3;
	}
	label_char[(int) '_']    = 4;
	function_char[(int) '_'] = 4;
	label_char[(int) '.']    = 5;
	function_char[(int) '.'] = 5;
	function_char[(int) '$'] = 6;
}

char *
end_symbol_name(char *P)
{
	char	*P1;
	int		c;

	c = *P;
	if (c == '_' || isalpha(c)) {
		for (P1 = P; function_char[(int) *P1]; ++P1);
		return(P1);
	}
	return(0);
}

/* Convert the pathP to an absolute path.
 */

char	*cwdP = 0;

char *
absolute_path(char *pathP)
{
	static char	full_path[2048];

	char	*P, *P1;

	P = full_path;
	if (*pathP == '/') {
		/* If the path is an absolute path we are done */
		strcpy(full_path, pathP);
	} else if (!cwdP || cwdP[0] != '/') {
		/* Don't know what the cwd is */
		strcpy(P, pathP);
	} else {
		strcpy(P, cwdP);
		P += strlen(cwdP);
		*P++ = '/';
		strcpy(P, pathP);
	}

	for (P = full_path; (P1 = strchr(P, '.')); ) {
		switch (P1[1]) {
		case '/':
			if (P1 == full_path || P1[-1] == '/') {
				// Erase ./
				P = P1;
				strcpy(P, P+2);
				continue;
			}
			break;
		case '.':
			if (P1[2] == '/' && P1 > full_path+1 && P1[-1] == '/') {
				// Remove <parent>/../
				for (P = P1-2; P != pathP && P[-1] != '/'; --P);
				strcpy(P, P1+3);
				continue;
			}
			break;
		}
		P = P1 + 1;
	}
	return(full_path);
}

void
put_html(const char *stringP, FILE *F)
{
	const char 	*P;
	char		c;

	for (P = stringP; (c = *P++); ) {
		switch (c) {
		case '>':
			fputs("&gt;", F);
			continue;
		case '<':
			fputs("&lt;", F);
			continue;
		case '&':
			fputs("&amp;", F);
			continue;
		case '\n':
			fputs("\n<br>\n", F);
			continue;
		}
		fputc(c, F);
}	}

FILE *
open_file(const char *filenameP)
{
	FILE	*F;

	F = fopen(filenameP, "w");
	if (!F) {
		fprintf(stderr, "Can't open %s\n", filenameP);
		exit(1);
	}
	return(F);
}
